package com.myoa.engineering.crawl.shopping.configuration.feign;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.myoa.engineering.crawl.shopping.dto.ExceptionMessage;
import feign.Response;
import feign.codec.ErrorDecoder;
import io.undertow.util.BadRequestException;
import javassist.NotFoundException;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.io.InputStream;

@Slf4j
public class FmkoreaClientErrorDecoder implements ErrorDecoder {

    private final ErrorDecoder errorDecoder = new Default();

    @Override
    public Exception decode(String methodsKey, Response response) {
        ExceptionMessage message = null;
        try (InputStream bodyIs = response.body()
                                          .asInputStream()) {
            ObjectMapper mapper = new ObjectMapper();
            message = mapper.readValue(bodyIs, ExceptionMessage.class);
        } catch (IOException e) {
            return new Exception(e.getMessage());
        }
        switch (response.status()) {
            case 400:
                return new BadRequestException(message.getMessage() != null ? message.getMessage() : "Bad Request");
            case 404:
                return new NotFoundException(message.getMessage() != null ? message.getMessage() : "Not found");
            default:
                return errorDecoder.decode(methodsKey, response);
        }

    }
}

