package com.myoa.engineering.crawl.shopping.crawlhandler;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.CollectionType;
import com.myoa.engineering.crawl.shopping.util.ObjectMapperFactory;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Base64;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public final class FmkoreaFake430Resolver {

    private static final Pattern PATTERN_DOCUMENT = Pattern.compile("var .+?\\s*=\\s*(\\[.+?\\];)");
    private static final Pattern PATTERN_COOKIE = Pattern.compile("escape\\('(.+?)'\\)");
    private static final ObjectMapper MAPPER = ObjectMapperFactory.DEFAULT_MAPPER;
    private static final CollectionType COLLECTION_TYPE = MAPPER.getTypeFactory().constructCollectionType(List.class, String.class);

    private static final DateTimeFormatter DATE_TIME_FORMATTER_COOKIE_LIFE = DateTimeFormatter.ofPattern("EEE, dd MMM yyyy HH:mm:ss 'GMT'", Locale.ENGLISH);

    private FmkoreaFake430Resolver() {
    }

    public static String resolveFake430(String fakeHtml) {
        Document parse = Jsoup.parse(fakeHtml);
        String javascript = parse.select("script").html();
        String cookieHtml = extractEncodedCookieHtml(javascript);

        try {
            List<String> chunks = MAPPER.readValue(cookieHtml, COLLECTION_TYPE);
            String decodedhtml = decodeHtmlChunks(chunks);
            String cookieHexValue = extractCookieHexValue(decodedhtml);
            return generateLiteTimeCookie(cookieHexValue);
        } catch (Exception e) {
            return null;
        }
    }

    private static String extractEncodedCookieHtml(String javascript) {
        final Matcher matcher = PATTERN_DOCUMENT.matcher(javascript);

        if (matcher.find()) {
            return matcher.group(1);
        }
        return "";
    }

    private static String decodeHtmlChunks(List<String> chunks) {
        return chunks.stream()
                     .map(e -> new String(Base64.getDecoder().decode(e.substring(3, e.length() - 3))))
                     .map(e -> (char) (e.charAt(0) - 3 + 256) % 256)
                     .collect(StringBuilder::new, StringBuilder::appendCodePoint, StringBuilder::append).toString();
    }

    private static String extractCookieHexValue(String decodedHtml) {
        final Matcher matcher = PATTERN_COOKIE.matcher(decodedHtml);
        if (matcher.find()) {
            return matcher.group(1);
        }
        return "";
    }

    private static String escape(String input) {
        StringBuilder result = new StringBuilder();
        for (char ch : input.toCharArray()) {
            if (Character.isLetterOrDigit(ch) || ch == '-' || ch == '_' || ch == '.' || ch == '~') {
                result.append(ch);
            } else {
                result.append(String.format("%%%02X", (int) ch));
            }
        }
        return result.toString();
    }

    public static String generateLiteTimeCookie(String cookieHexValue) {
        LocalDateTime ldt = LocalDateTime.now().plusDays(1L);

        String cookie = "lite_year=" + escape(cookieHexValue) +
                "; expires=" + ldt.format(DATE_TIME_FORMATTER_COOKIE_LIFE) + "; path=/";
        return cookie;
    }

}
