package com.myoa.engineering.crawl.shopping.service;

import com.myoa.engineering.crawl.shopping.domain.entity.v2.SubscribedKeyword;
import com.myoa.engineering.crawl.shopping.domain.model.v2.SubscribedKeywordAggregatedModel;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
public class SubscribedKeywordCacheService {

    private final SubscribedKeywordQueryService subscribedKeywordQueryService;

    public SubscribedKeywordCacheService(SubscribedKeywordQueryService subscribedKeywordQueryService) {
        this.subscribedKeywordQueryService = subscribedKeywordQueryService;
    }

    @Cacheable(cacheNames = "subscribe.keywords", key = "#slackId + '_' + #crawlTarget.name()")
    public SubscribedKeywordAggregatedModel getSubscribedKeywordsCached(String slackId, CrawlTarget crawlTarget) {
        System.out.println("getSubscribedKeywordsCached");
        List<String> keywords = subscribedKeywordQueryService.findByUserWithTarget(slackId, crawlTarget)
                                                             .stream().map(SubscribedKeyword::getKeyword).toList();
        return SubscribedKeywordAggregatedModel.of(slackId, crawlTarget, keywords);
    }

    @Cacheable(cacheNames = "subscribe.keywords", key = "#slackId")
    public Map<CrawlTarget, SubscribedKeywordAggregatedModel> getSubscribedKeywordsCached(String slackId) {
        System.out.println("getSubscribedKeywordsCached");
        return subscribedKeywordQueryService.findByUser(slackId)
                                            .stream()
                                            .collect(Collectors.groupingBy(SubscribedKeyword::getCrawlTarget,
                                                                           Collectors.mapping(SubscribedKeyword::getKeyword, Collectors.toList())))
                                            .entrySet().stream()
                                            .collect(Collectors.toMap(Map.Entry::getKey, e -> SubscribedKeywordAggregatedModel.of(slackId, e.getKey(), e.getValue())));
    }

}
