package com.myoa.engineering.crawl.shopping.service.slack.shopping;

import com.myoa.engineering.crawl.shopping.domain.entity.v2.SubscribedKeyword;
import com.myoa.engineering.crawl.shopping.dto.feed.v1.SubscribedKeywordDTO;
import com.myoa.engineering.crawl.shopping.dto.slack.v2.SlackSlashCommandDTO;
import com.myoa.engineering.crawl.shopping.service.SubscribedKeywordCommandService;
import com.myoa.engineering.crawl.shopping.service.slack.ShoppingCommandProcessor;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

@Component
public class KeywordShoppingCommandProcessor implements ShoppingCommandProcessor {

    private final SubscribedKeywordCommandService subscribedKeywordCommandService;

    public KeywordShoppingCommandProcessor(SubscribedKeywordCommandService subscribedKeywordCommandService) {
        this.subscribedKeywordCommandService = subscribedKeywordCommandService;
    }

    @Override
    public boolean isAssignable(String menuContext) {
        return menuContext.equals("키워드");
    }

    @Override
    public String process(SlackSlashCommandDTO requestDTO) {
        // "키워드 추가 (쇼핑몰명) (키워드명)"
        final String[] splited = requestDTO.getText().split("\\s+", 4);

        final String action = splited[1];

        switch (action) {
            case "추가" -> {
                final CrawlTarget crawlTarget = CrawlTarget.fromAlias(splited[2]);
                final String keyword = splited[3];
                SubscribedKeyword subscribedKeyword =
                        SubscribedKeyword.builder()
                                         .crawlTarget(crawlTarget)
                                         .userId(requestDTO.getUser_id())
                                         .keyword(keyword)
                                         .build();
                subscribedKeywordCommandService.add(subscribedKeyword);
                return "keyword: [" + keyword + "] 추가완료";
            }
            case "삭제" -> {
                final CrawlTarget crawlTarget = CrawlTarget.fromAlias(splited[2]);
                final String keyword = splited[3];
                subscribedKeywordCommandService.delete(requestDTO.getUser_id(), crawlTarget, keyword);
                return "keyword: [" + keyword + "] 삭제완료";
            }
            case "목록" -> {
                List<SubscribedKeywordDTO> keywords = subscribedKeywordCommandService.list(requestDTO.getUser_id());
                if (keywords.isEmpty()) {
                    return "등록된 키워드가 없습니다.";
                }
                return keywords.stream().map(SubscribedKeywordDTO::toMessage)
                               .collect(Collectors.joining("\n"));
            }
            default -> {
                return "지원하지 않는 명령어입니다.";
            }
        }
    }
}
