package com.myoa.engineering.crawl.shopping.util;

import org.springframework.lang.Nullable;

import java.time.*;
import java.time.format.DateTimeFormatter;

public final class DateTimeUtils {

    public static final DateTimeFormatter FORMATTER_HHMMss = DateTimeFormatter.ofPattern("HH:mm:ss");
    public static final DateTimeFormatter FORMATTER_YYMMDD_SLASH = DateTimeFormatter.ofPattern("yy/MM/dd");
    public static final DateTimeFormatter FORMATTER_YYMMDD_DOT = DateTimeFormatter.ofPattern("yyyy.MM.dd");
    public static final DateTimeFormatter FORMATTER_HHMM = DateTimeFormatter.ofPattern("HH:mm");
    private static final ZoneId ZONE_ASIA_SEOUL = ZoneId.of("Asia/Seoul");

    private DateTimeUtils() {
    }

    public static ZonedDateTime parse(String dateTimeString, DateTimeFormatter formatter, @Nullable DateTimeFormatter fallback) {
        try {
            LocalTime time = LocalTime.parse(dateTimeString, formatter);
            LocalDateTime dateTime = LocalDateTime.of(LocalDate.now(), time);
            if (dateTime.isAfter(LocalDateTime.now())) {
                dateTime = dateTime.minusDays(1);
            }
            return dateTime.atZone(ZONE_ASIA_SEOUL);
        } catch (Exception ignored) {
        }

        try {
            LocalDate date = LocalDate.parse(dateTimeString, fallback);
            return date.atStartOfDay(ZONE_ASIA_SEOUL);
        } catch (Exception ignored) {
        }
        return null;
    }

}
